@@ -5,19 +5,11 @@
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
 # by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Daniel Gerber [ctb], Kaspar Moesinger [ctb],           #
-#    Youcef Saad [ctb] (SPARSEKIT),                         #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines),      #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines)       #
+#    Roman Flury [ctb], Daniel Gerber [ctb],                #
+#    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
-     
+
 
 ########################################################################
 ########################################################################
@@ -27,10 +19,10 @@
 #    determinant                (the later because it is based on chol)
 #
 # As well as associated S4 elements.
-# 
+#
 # The key element is a new class: "spam.chol.NgPeyton", the output
 # of 'chol'
-# 
+#
 ########################################################################
 ########################################################################
 
@@ -50,7 +42,7 @@ setClass("spam.chol.NgPeyton",
 # xsuper=supernodes
 # c(... nnztmp,cachesize)= memory
 
-  
+
 #setClass("spam.chol.NgPeyton",
 #         representation(nrow="integer",nnzlindx="integer",
 #                        nsuper="integer",lindx="integer",xlindx="integer",nnzl="integer",
@@ -111,7 +103,7 @@ setMethod("c","spam.chol.NgPeyton", function(x,...){
     ## xcolindices <- .Fortran('calcja',
     ##                         as.integer(nrow),
     ##                         as.integer(nsuper),
-    ##                         as.integer(x@supernodes), 
+    ##                         as.integer(x@supernodes),
     ##                         as.integer(x@colindices),
     ##                         as.integer(x@colpointers),
     ##                         as.integer(x@rowpointers),
@@ -122,19 +114,19 @@ setMethod("c","spam.chol.NgPeyton", function(x,...){
         SS <- .format64()
     else
         SS <- .format32
-    
+
     xcolindices <- .C64('calcja',
                         SIGNATURE = rep(SS$signature, 7),
-                        
+
                         nrow,
                         nsuper,
                         x@supernodes,
                         x@colindices,
-                        
+
                         x@colpointers,
                         x@rowpointers,
                         xja = vector_dc( SS$type, nnzR),
-                        
+
                         INTENT=c("r", "r", "r", "r",
                                  "r", "r", "w"),
                         NAOK = getOption("spam.NAOK"),
@@ -142,12 +134,12 @@ setMethod("c","spam.chol.NgPeyton", function(x,...){
     cx <- .C64("spamcsrdns",
                SIGNATURE = c(SS$signature, "double" , SS$signature, SS$signature,
                              "double"),
-               
+
                nrow = nrow,
                entries = x@entries,
                colindices = xcolindices,
                rowpointers = x@rowpointers,
-               
+
                res = vector_dc( "double", nrow*nrow),
 
                INTENT = c("r", "r", "r", "r",
@@ -165,28 +157,28 @@ as.spam.chol.NgPeyton <- function(x, eps = getOption("spam.eps")) {
         SS <- .format64()
     else
         SS <- .format32
-    
+
   if (eps<.Machine$double.eps) stop("'eps' should not be smaller than machine precision",call.=FALSE)
   nrow <- x@dimension[1]
   nnzR <- x@rowpointers[nrow+1]-1
   nsuper <- length(x@supernodes)-1
-  
+
   colindices <- .C64('calcja',
                      SIGNATURE=rep(SS$signature, 7),
-                     
+
                      nrow,
                      nsuper,
                      x@supernodes,
                      x@colindices,
-                     
+
                      x@colpointers,
                      x@rowpointers,
                      xja=vector(SS$type, nnzR), #!1#
-                     
+
                      INTENT=c("r", "r", "r", "r", "r", "r", "w"),
                      NAOK = getOption("spam.NAOK"),
                      PACKAGE = SS$package)$xja
-  
+
   return(.newSpam(
     entries=x@entries,
     colindices=colindices,
@@ -208,7 +200,7 @@ setMethod("as.spam","spam.chol.NgPeyton", as.spam.chol.NgPeyton)
 #"backsolve.default" <- base::backsolve
 #setGeneric("backsolve")
 #setMethod("backsolve","matrix",base::backsolve)
-  
+
 #"forwardsolve" <- function(l,x, ...) UseMethod("forwardsolve")
 #"forwardsolve.default" <- base::forwardsolve
 #setGeneric("forwardsolve")
@@ -229,7 +221,7 @@ setGeneric("forwardsolve", def = function(l, x, ...) standardGeneric("forwardsol
 #           signature = c("l", "x"))#, where = where)
 ##### setGenericImplicit("forwardsolve")#, restore=FALSE)
 
-  
+
 "ordering.default" <- function(x,inv=FALSE) stop('Operation not defined form this class')
 
 #ordering <- function(x,...) stop('Operation not defined form this class')
@@ -262,7 +254,7 @@ update.spam.chol.NgPeyton <- function(object,x,...){
   if (!is.spam(x))
     stop("Covariance should be a 'spam' object.")
   if ((x@rowpointers[nrow+1]-1) != object@nnzA)
-    stop("Updated covariance entries do not match length of original one.") 
+    stop("Updated covariance entries do not match length of original one.")
 
   ## u <- .Fortran("updatefactor",
   ##               as.integer(nrow),
@@ -276,16 +268,16 @@ update.spam.chol.NgPeyton <- function(object,x,...){
   ##               rowpointers = as.integer(object@rowpointers),#xlnz
   ##               snode=as.integer(object@snmember),  xsuper=as.integer(object@supernodes),
   ##               cachesize=as.integer(object@memory[3]),
-  ##               ierr = 0L,         
+  ##               ierr = 0L,
     ##               NAOK = getOption("spam.NAOK"),PACKAGE="spam")
     if( getOption("spam.force64") || .format.spam(object)$package != "spam" || .format.spam(x)$package != "spam" )
         SS <- .format64()
     else
         SS <- .format32
-    
+
     u <- .C64("updatefactor",
      ##          subroutine updatefactor( m,nnzd,
-     ## &     d,jd,id, invp,perm, 
+     ## &     d,jd,id, invp,perm,
      ## &                lindx,xlindx, nsuper,lnz,xlnz,
      ## &                snode, xsuper,
      ## &                cachesize,ierr)
@@ -295,17 +287,17 @@ update.spam.chol.NgPeyton <- function(object,x,...){
                             "double", SS$signature, SS$signature, SS$signature, SS$signature, SS$signature ),
               nrow,
               object@nnzA,
-              
+
               d =  x@entries,
               jd = x@colindices,
               id = x@rowpointers,
-              
+
               object@invpivot,
               object@pivot,
               lindx = object@colindices,
               xlindx = object@colpointers,
               nsuper = length(object@supernodes)-1,
-              
+
               entries = vector_dc( "double", length(object@entries)), #lnz
               rowpointers = object@rowpointers,#xlnz
               snode = object@snmember,
@@ -313,10 +305,10 @@ update.spam.chol.NgPeyton <- function(object,x,...){
               cachesize = object@memory[3],
               ierr = 0,
 
-              ## INTENT = c("r", "r", 
+              ## INTENT = c("r", "r",
               ##            "r", "r", "r",
               ##            "r", "r", "r", "r", "r",
-              ##            "rw", "rw", "r", "r", "rw", "rw"), 
+              ##            "rw", "rw", "r", "r", "rw", "rw"),
               NAOK = getOption("spam.NAOK"),
               PACKAGE = SS$package)
 
@@ -343,14 +335,14 @@ chol.spam <- function(x, pivot = "MMD",
                       method="NgPeyton",
                       memory=list(),
                       eps = getOption("spam.eps"), ...){
-    
+
     force64 <- getOption("spam.force64")
   if(force64 || prod(dim(x)) > 2147483647)
       SS <- .format64()
   else
       SS <- .format32
-  #!3# 
-  
+  #!3#
+
 
   if (eps<.Machine$double.eps) stop("'eps' should not be smaller than machine precision",call.=FALSE)
   nrow <- x@dimension[1]
@@ -362,15 +354,15 @@ chol.spam <- function(x, pivot = "MMD",
 # base rule:
 #  `nrow(x) * .Machine$double.neg.eps * max(diag(x)`.
 
-  
+
   if(getOption("spam.cholsymmetrycheck")) {
-    test <- isSymmetric(x, tol = eps*100) 
-#  from help of isSymmetric:  
+    test <- isSymmetric(x, tol = eps*100)
+#  from help of isSymmetric:
 #     isSymmetric(object, tol = 100 * .Machine$double.eps, ...)
     if (!isTRUE(test))
       stop("Input matrix to 'chol' not symmetric (up to 100*eps)",call.=FALSE)
   }
-  
+
   if (method != "NgPeyton")
     warning(gettextf("method = '%s' is not supported. Using 'NgPeyton'",
                      method), domain = NA)
@@ -398,7 +390,7 @@ chol.spam <- function(x, pivot = "MMD",
   ### IMPROVEME get better parameter values
   nnzcfact <- c(5,1,5)
   nnzRfact <- c(5,1,2)
-  # nnzcolindices = length of array holding the colindices 
+  # nnzcolindices = length of array holding the colindices
   if(is.null(memory$nnzcolindices))  {
     nnzcolindices <- ifelse((nnzA/nrow < 5), # very sparse matrix
                             max(1000,nnzA*(1.05*nnzA/nrow-3.8)),
@@ -408,27 +400,27 @@ chol.spam <- function(x, pivot = "MMD",
     nnzcolindices <- max(memory$nnzcolindices,nnzA)
     memory$nnzcolindices <- NULL
   }
-  # nnzR = length of array holding the nonzero values of the factor 
+  # nnzR = length of array holding the nonzero values of the factor
   if(is.null(memory$nnzR)) {
     nnzR <- min(max(4*nnzA,floor(.2*nnzA^1.3))*nnzRfact[doperm+1],nrow*(nrow+1)/2)
   } else {
     nnzR <- memory$nnzR
     memory$nnzR <- NULL
   }
-  
+
 #!4#
   if(is.null(memory$cache))
     cache <- 512
   else {
-    cache <- memory$cache 
+    cache <- memory$cache
     memory$cache <- NULL
   }
 
   if (length( memory)>0 )
     warning("The component(s) ", paste("'",names(memory),"'",sep='',collapse=","),
             " of the argument 'memory'\npassed to function 'chol' not meaningful and hence ignored.",call.=FALSE)
-   
-  cholstep.intent <- c("r", "r", "r", "r", 
+
+  cholstep.intent <- c("r", "r", "r", "r",
                     "r", "r", "rw", "rw",
                     "rw", "r", "rw", "rw",
                     "rw", "rw", "rw", "rw",
@@ -436,7 +428,7 @@ chol.spam <- function(x, pivot = "MMD",
 
   cholstep.signature <- rep(SS$signature, 20)
   cholstep.signature[c(3,15)] <- "double"
-   
+
   z <- .C64("cholstepwise",
             SIGNATURE=cholstep.signature,
 
@@ -444,32 +436,32 @@ chol.spam <- function(x, pivot = "MMD",
             nnzA = nnzA,
             d =  x@entries,
             jd = x@colindices,
-            
+
             id = x@rowpointers, #5
             doperm = doperm,
             invp = vector(SS$type,nrow),
             perm = pivot,
-            
-            nnzlindx = 1L, #9       
+
+            nnzlindx = 1L, #9
             nnzcolindices = nnzcolindices,
-            lindx = vector(SS$type, nnzcolindices),   
+            lindx = vector(SS$type, nnzcolindices),
             xlindx = vector(SS$type, nrow+1),
-            
+
             nsuper = 1L, #13
             nnzR = nnzR,
             lnz = vector("double",nnzR),
-            xlnz = vector(SS$type, nrow+1), 
-                
+            xlnz = vector(SS$type, nrow+1),
+
             snode = vector(SS$type, nrow), #17
-            xsuper = vector(SS$type, nrow+1),   
+            xsuper = vector(SS$type, nrow+1),
             cachesize = cache,
             ierr = 0L,
-            
+
             INTENT=cholstep.intent,
             NAOK = getOption("spam.NAOK"),
             PACKAGE = SS$package)
 
-  if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.") 
+  if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.")
   if(z$ierr == 6) stop("Inconsitency in the input",call.=FALSE)
 
   while( z$ierr>1) {
@@ -487,44 +479,44 @@ chol.spam <- function(x, pivot = "MMD",
     }
     z <- .C64("cholstepwise",
               SIGNATURE=cholstep.signature,
-              
+
               nrow = nrow,
               nnzA = x@rowpointers[nrow+1]-1,
               d =  x@entries,
               jd = x@colindices,
-              
+
               id = x@rowpointers,
               doperm = doperm,
               invp = vector(SS$type,nrow),
               perm = pivot,
-              
-              nnzlindx = vector(SS$type,1),             
+
+              nnzlindx = vector(SS$type,1),
               nnzcolindices = nnzcolindices,
-              lindx = vector(SS$type, nnzcolindices),     
+              lindx = vector(SS$type, nnzcolindices),
               xlindx = vector(SS$type, nrow+1),     #
-              
+
               nsuper = 1,          #
               nnzR = nnzR,#
               lnz = vector("double",nnzR),        #
               xlnz = vector(SS$type, nrow+1),     #
-              
+
               snode = vector(SS$type, nrow),
-              xsuper = vector(SS$type, nrow+1),   
+              xsuper = vector(SS$type, nrow+1),
               cachesize = cache,
-              ierr = 0L,       
-              
+              ierr = 0L,
+
               INTENT=cholstep.intent,
               NAOK = getOption("spam.NAOK"),
               PACKAGE = SS$package)
-    
-    if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.") 
+
+    if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.")
   }
   nnzR <- z$xlnz[length(z$xlnz)]-1
 
   newx <- new("spam.chol.NgPeyton")
   slot(newx,"entries",check=FALSE) <- z$lnz[1:nnzR]
   slot(newx,"colindices",check=FALSE) <- z$lindx[1:z$nnzlindx]
-  slot(newx,"colpointers",check=FALSE) <- z$xlindx[1:(z$nsuper+1)] 
+  slot(newx,"colpointers",check=FALSE) <- z$xlindx[1:(z$nsuper+1)]
   slot(newx,"rowpointers",check=FALSE) <- z$xlnz
   slot(newx,"dimension",check=FALSE) <- c(nrow,nrow)
   slot(newx,"pivot",check=FALSE) <- z$perm
@@ -549,10 +541,10 @@ solve.spam <- function (a, b,  Rstruct = NULL, ...) {
   }
   p <- dim(b)[2]
   if(nrow!=dim(b)[1])stop("'b' must be compatible with 'a'")
-  
+
   # if we have a spam matrix, we calculate the Cholesky factor
   if (is(a,"spam"))
-    if (is(Rstruct, "spam.chol.NgPeyton")) 
+    if (is(Rstruct, "spam.chol.NgPeyton"))
         a <- update.spam.chol.NgPeyton(Rstruct, a, ...)
     else a <- chol.spam(a, ...)
 
@@ -574,11 +566,11 @@ solve.spam <- function (a, b,  Rstruct = NULL, ...) {
           SS <- .format64()
       else
           SS <- .format32
-      
+
       z <- .C64("backsolves",
                 SIGNATURE = c(rep(SS$signature,5),"double", rep(SS$signature, 4),
                               rep("double",3)),
-                
+
                 m = nrow,
                 nsuper,
                 p,
@@ -589,17 +581,17 @@ solve.spam <- function (a, b,  Rstruct = NULL, ...) {
                 a@invpivot,
                 a@pivot,
                 a@supernodes,
-                
+
                 vector_dc("double",nrow),
                 sol = vector_dc("double",nrow*p),
                 as.vector(b,"double"),
 
-                INTENT = c( rep( "r", 10), rep( "rw", 3)), 
+                INTENT = c( rep( "r", 10), rep( "rw", 3)),
                 NAOK = getOption("spam.NAOK"),
                 PACKAGE = SS$package )$sol
   } else z <- backsolve(a, forwardsolve( t(a),b))
     # see the helpfile for a comment about the 't(a)' construct.
-  
+
   if ( p!=1)    dim(z) <- c(nrow,p)
   return( z)
 }
@@ -607,12 +599,12 @@ solve.spam <- function (a, b,  Rstruct = NULL, ...) {
 chol2inv.spam <- function (x, ...) {
     ## print("chol2inv.spam")
     nrow <- x@dimension[1]
-    
+
     if (is(x,"spam.chol.NgPeyton")) {
         ## print("x = spam.chol.NgPeyton")
         y <- vector("double",nrow*nrow)
         y[1L + 0L:(nrow - 1L) * (nrow + 1L)] <- 1.0
-    
+
         ## z <- .Fortran("backsolves", m = nrow,
         ##               as.integer( length(x@supernodes)-1), as.integer(nrow), as.integer(x@colindices),
         ##               as.integer(x@colpointers), as.double(x@entries),
@@ -620,20 +612,20 @@ chol2inv.spam <- function (x, ...) {
         ##               as.integer(x@supernodes), vector("double",nrow), sol = vector("double",nrow*nrow), as.double(y),
         ##               NAOK = getOption("spam.NAOK"),
         ##               PACKAGE = "spam")$sol
-    
+
         if( getOption("spam.force64") || .format.spam(x)$package !="spam" )
             SS <- .format64()
         else
             SS <- .format32
-   
+
         ## print(SS$package)
-        
+
         z <- .C64("backsolves",
           ##         subroutine backsolves(m,nsuper,nrhs,lindx,xlindx,lnz,
      ## &                   xlnz,invp,perm,xsuper,newrhs,sol,b)
                   SIGNATURE = c( rep( SS$signature, 5), "double", rep( SS$signature, 4),
                                 rep( "double", 3)),
-                  
+
                   m = nrow, #r
                   length(x@supernodes)-1, #r
                   nrow, #r
@@ -644,7 +636,7 @@ chol2inv.spam <- function (x, ...) {
                   x@invpivot, #r
                   x@pivot, #r
                   x@supernodes, #r
-                  
+
                   vector_dc("double",nrow), #rw
                   sol = vector_dc("double",nrow*nrow), #w
                   y, #r
@@ -653,7 +645,7 @@ chol2inv.spam <- function (x, ...) {
                              "rw", "w", "r"),
                   NAOK = getOption("spam.NAOK"),
                   PACKAGE = SS$package)$sol
-        
+
         dim(z) <- c(nrow,nrow)
     } else z <- backsolve.spam(x, forwardsolve.spam( t(x), diag(nrow)))
     return( z)
@@ -662,13 +654,13 @@ chol2inv.spam <- function (x, ...) {
 backsolve.spam <- function(r, x,...){#, k = NULL, upper.tri = NULL, transpose = NULL){
 # r: spam.chol.NgPeyton structure as returned by chol.spam or a spam object
 # x: rhs a vector or a matrix in dense form
-# dimensions:  ( m x n) ( n x p) 
+# dimensions:  ( m x n) ( n x p)
 
     if( getOption("spam.force64") || .format.spam(r)$package !="spam" )
         SS <- .format64()
     else
         SS <- .format32
-  
+
 
   m <- r@dimension[1]
   if(is.vector(x)) {
@@ -676,8 +668,8 @@ backsolve.spam <- function(r, x,...){#, k = NULL, upper.tri = NULL, transpose =
     p <- 1
   } else {
     x <- as.matrix(x)
-    n <- nrow(x)
-    p <- ncol(x)
+    n <- dim(x)[1]
+    p <- dim(x)[2]
   }
 
   # we separate between "spam.chol.NgPeyton" and "spam"
@@ -685,28 +677,28 @@ backsolve.spam <- function(r, x,...){#, k = NULL, upper.tri = NULL, transpose =
     if (n!=m) stop("Cholesky factor 'r' not compatible with 'x'")
     nsuper <- length(r@supernodes)-1
     if (!getOption("spam.dopivoting")) {
-      
+
       z <- .C64("backsolvef",
-                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature, 
-                            SS$signature, "double", SS$signature, SS$signature, 
+                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature,
+                            SS$signature, "double", SS$signature, SS$signature,
                             "double"),
-                
+
                 m,
                 nsuper,
                 p,
                 r@colindices,
-                
+
                 r@colpointers,
                 r@entries,
-                r@rowpointers, 
+                r@rowpointers,
                 r@supernodes,
-                
+
                 sol = vector("double",m*p),
-                
+
                 INTENT=c("r", "r", "r", "r", "r", "r", "r", "r", "w"),
                 NAOK = getOption("spam.NAOK"),
                 PACKAGE=SS$package)$sol
-                    
+
     }else{
       z <- .C64("pivotbacksolve",
                 SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature,
@@ -718,21 +710,21 @@ backsolve.spam <- function(r, x,...){#, k = NULL, upper.tri = NULL, transpose =
                 nsuper,
                 p,
                 r@colindices,
-                
+
                 r@colpointers,
                 r@entries,
                 r@rowpointers,
                 r@invpivot,
-                
+
                 r@pivot,
                 r@supernodes,
                 vector("double",m),
                 sol = vector("double",m*p),
-                
+
                 x,
-                
+
                 INTENT=c("r", "r", "r", "r",
-                    "r", "r", "r", "r", 
+                    "r", "r", "r", "r",
                     "r", "r", "r", "w",
                     "r"),
                 NAOK = getOption("spam.NAOK"),
@@ -742,26 +734,26 @@ backsolve.spam <- function(r, x,...){#, k = NULL, upper.tri = NULL, transpose =
     if (n!=m) stop("Triangular matrix 'r' not compatible with 'x'")
     # solve R sol = x
     z <- .C64("spamback",
-              SIGNATURE=c(SS$signature, SS$signature, "double", "double", 
+              SIGNATURE=c(SS$signature, SS$signature, "double", "double",
                           "double", SS$signature, SS$signature),
               m=m,
               unused=p,
               sol = vector("double",m*p),
               x=x,
-       
+
               al=r@entries,
               jal=r@colindices,
               ial=r@rowpointers,
-              
+
               INTENT=c("rw", "rw", "rw", "rw", "rw", "rw", "rw"),
               NAOK = getOption("spam.NAOK"),
               PACKAGE=SS$package)
     if (z$m<0) stop(gettextf("singular matrix in 'backsolve'. Last zero in diagonal [%d]",
             -z$m), domain = NA)
      else z <- z$sol
-   
+
   }
-  
+
   if (p>1) dim(z) <- c(m,p)
   return(z)
 }
@@ -777,15 +769,15 @@ forwardsolve.spam <- function(l, x,...){#, k = NULL, upper.tri = NULL, transpose
         SS <- .format64()
     else
         SS <- .format32
-    
+
   m <- l@dimension[1]
   if(is.vector(x)) {
     n <- length(x)
     p <- 1L
   } else {
     if(!is.matrix(x)) x <- as.matrix(x)
-    n <- nrow(x)
-    p <- ncol(x)
+    n <- dim(x)[1]
+    p <- dim(x)[2]
   }
 
   # we separate between "spam.chol.NgPeyton" and "spam"
@@ -794,22 +786,22 @@ forwardsolve.spam <- function(l, x,...){#, k = NULL, upper.tri = NULL, transpose
     nsuper <- length(l@supernodes)-1
     if (!getOption("spam.dopivoting")) {
       z <- .C64("forwardsolvef",
-                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature, 
-                            SS$signature, "double", SS$signature, SS$signature, 
+                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature,
+                            SS$signature, "double", SS$signature, SS$signature,
                             "double"),
 
                 m,
                 nsuper,
                 p,
                 l@colindices,
-                
+
                 l@colpointers,
                 l@entries,
-                l@rowpointers, 
+                l@rowpointers,
                 l@supernodes,
-                
+
                 sol = vector("double",m*p),
-                
+
                 INTENT=c("r", "r", "r", "r",
                     "r", "r", "r", "r",
                     "w"),
@@ -817,29 +809,29 @@ forwardsolve.spam <- function(l, x,...){#, k = NULL, upper.tri = NULL, transpose
                 PACKAGE=SS$package)$sol
     }else{
       z <- .C64("pivotforwardsolve",
-                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature, 
-                            SS$signature, "double", SS$signature, SS$signature, 
-                            SS$signature, SS$signature, "double", "double", 
+                SIGNATURE=c(SS$signature, SS$signature, SS$signature, SS$signature,
+                            SS$signature, "double", SS$signature, SS$signature,
+                            SS$signature, SS$signature, "double", "double",
                             "double"),
                 m,
                 nsuper,
                 p,
                 l@colindices,
-                
+
                 l@colpointers,
                 l@entries,
                 l@rowpointers,
                 l@invpivot,
-                
+
                 l@pivot,
                 l@supernodes,
                 vector("double",m), #!5#
                 sol = vector("double",m*p),
-                
+
                 x,
-                
+
                 INTENT=c("r", "r", "r", "r",
-                    "r", "r", "r", "r", 
+                    "r", "r", "r", "r",
                     "r", "r", "r", "w",
                     "r"),
                 PACKAGE=SS$package)$sol
@@ -848,18 +840,18 @@ forwardsolve.spam <- function(l, x,...){#, k = NULL, upper.tri = NULL, transpose
     if (n!=m) stop("Triangular matrix 'l' not compatible with 'x'")
     # solve L sol = x
     z <- .C64("spamforward",
-              SIGNATURE=c(SS$signature, SS$signature, "double", "double", 
+              SIGNATURE=c(SS$signature, SS$signature, "double", "double",
                           "double", SS$signature, SS$signature),
 
               m=m,
               p,
               sol = vector("double",m*p),
               x=x,
-              
+
               al=l@entries,
               jal=l@colindices,
               ial=l@rowpointers,
-              
+
               INTENT=c("rw", "r", "w", "r", "r", "r", "r"),
               NAOK = getOption("spam.NAOK"),
               PACKAGE=SS$package)
@@ -891,20 +883,23 @@ setMethod("forwardsolve","spam.chol.NgPeyton", forwardsolve.spam)
 determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton",
                               memory=list(),eps = getOption("spam.eps"), ...){
    # print("determinant.spam")
-    
+
   if (eps<.Machine$double.eps) stop("'eps' should not be smaller than machine precision",call.=FALSE)
   logdet <- list()
- #### start from above 
+ #### start from above
   nrow <- x@dimension[1]
   nnzA <- as.integer( x@rowpointers[nrow+1]-1)
   if(nrow!=x@dimension[2]) stop("non-square matrix in 'chol'",call.=FALSE)
 
+  if(nrow <= 1) {
+    return(determinant(as.matrix(x@entries))) }
+
   if(getOption("spam.cholsymmetrycheck")) {
-    test <- isSymmetric.spam(x, tol = eps*100) 
+    test <- isSymmetric.spam(x, tol = eps*100)
     if (!isTRUE(test))
       stop("Input matrix to 'chol' not symmetric (up to 100*eps)",call.=FALSE)
   }
-  
+
   if (method != "NgPeyton")
     warning(gettextf("method = '%s' is not supported. Using 'NgPeyton'",
                      method), domain = NA)
@@ -929,11 +924,11 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
   } else stop("'pivot' should be 'MMD', 'RCM' or a permutation")
 
 
-  #!6# 
+  #!6#
     nnzcfact <- c(5,1,5)
     nnzRfact <- c(5,1,2)
-    
-  # nnzcolindices = length of array holding the colindices 
+
+  # nnzcolindices = length of array holding the colindices
   if(is.null(memory$nnzcolindices))  {
     nnzcolindices <- ifelse((nnzA/nrow < 5), # very sparse matrix
                             max(1000,nnzA*(1.05*nnzA/nrow-3.8)),
@@ -943,13 +938,13 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
     nnzcolindices <- max(memory$nnzcolindices,nnzA)
     memory$nnzcolindices <- NULL
   }
-  # nnzR = length of array holding the nonzero values of the factor 
+  # nnzR = length of array holding the nonzero values of the factor
   if(is.null(memory$nnzR))    nnzR <- min(max(4*nnzA,floor(.4*nnzA^1.2))*nnzRfact[doperm+1],nrow*(nrow+1)/2)  else {
     nnzR <- memory$nnzR
     memory$nnzR <- NULL
   }
   if(is.null(memory$cache))    cache <- 64  else {
-    cache <- memory$cache 
+    cache <- memory$cache
     memory$cache <- NULL
   }
 
@@ -961,32 +956,32 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
   ##               nrow = as.integer(nrow) ,nnzA = as.integer(x@rowpointers[nrow+1]-1),
   ##               d =  as.double(x@entries),jd = as.integer(x@colindices),id = as.integer(x@rowpointers),
   ##               doperm = as.integer(doperm), invp = vector("integer",nrow), perm = as.integer(pivot),
-  ##               nnzlindx = vector("integer",1),             
+  ##               nnzlindx = vector("integer",1),
   ##               nnzcolindices = as.integer(nnzcolindices),
-  ##               lindx = vector("integer",nnzcolindices),     
+  ##               lindx = vector("integer",nnzcolindices),
   ##               xlindx = vector("integer",nrow+1),     #
   ##               nsuper = vector("integer",1),          #
   ##               nnzR = as.integer(nnzR),#
   ##               lnz = vector("double",nnzR),        #
   ##               xlnz = vector("integer",nrow+1),     #
   ##               snode = vector("integer",nrow),
-  ##               xsuper = vector("integer",nrow+1),   
+  ##               xsuper = vector("integer",nrow+1),
   ##               cachesize = as.integer(cache),
-  ##               ierr = 0L,          
+  ##               ierr = 0L,
     ##               NAOK = getOption("spam.NAOK"), PACKAGE = "spam")
 
     if( getOption("spam.force64") || prod(dim(x)) > 2147483647)
         SS <- .format64()
     else
         SS <- .format32
-    cholstep.intent <- c("r", "r", "r", "r", 
+    cholstep.intent <- c("r", "r", "r", "r",
                     "r", "rw", "rw", "rw",
                     "rw", "rw", "rw", "rw",
                     "rw", "rw", "rw", "rw",
                     "rw", "rw", "rw", "rw")
     cholstep.signature <- rep(SS$signature, 20)
     cholstep.signature[c(3,15)] <- "double"
-    
+
     z <- .C64("cholstepwise",
       ##             subroutine cholstepwise(m,nnzd,
      ## &     d,jd,id,    doperm,invp,perm,
@@ -995,29 +990,29 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
      ## &                snode,xsuper,
      ## &                cachsz,ierr)
               SIGNATURE = cholstep.signature,
-              
+
               nrow = nrow ,
               nnzA = x@rowpointers[nrow+1]-1,
               d =  x@entries,
               jd = x@colindices,
-              
+
               id = x@rowpointers,
               doperm = doperm,
               invp = vector_dc( SS$type, nrow),
               perm = pivot,
-              
-              nnzlindx = vector_dc( SS$type, 1),             
+
+              nnzlindx = vector_dc( SS$type, 1),
               nnzcolindices = nnzcolindices,
-              lindx = vector_dc( SS$type, nnzcolindices),     
+              lindx = vector_dc( SS$type, nnzcolindices),
               xlindx = vector_dc( SS$type,nrow+1),     #
-              
+
               nsuper = vector_dc( SS$type,1),          #
               nnzR = nnzR,#
               lnz = vector_dc( "double", nnzR),        #
               xlnz = vector_dc( SS$type, nrow+1),     #
-              
+
               snode = vector_dc( SS$type, nrow),
-              xsuper = vector_dc( SS$type, nrow+1),   
+              xsuper = vector_dc( SS$type, nrow+1),
               cachesize = cache,
               ierr = 0,
 
@@ -1026,7 +1021,7 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
               PACKAGE = SS$package)
 
 
-  if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.") 
+  if(z$ierr == 1) stop("Singularity problem when calculating the Cholesky factor.")
   if(z$ierr == 6) stop("Inconsitency in the input",call.=FALSE)
 
   while( z$ierr>1) {
@@ -1045,24 +1040,24 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
                   nrow = nrow,nnzA = as.integer(x@rowpointers[nrow+1]-1),
                   d =  as.double(x@entries),jd = x@colindices,id = x@rowpointers,
                   doperm = doperm,invp = vector("integer",nrow), perm = pivot,
-                  nnzlindx = vector("integer",1),             
+                  nnzlindx = vector("integer",1),
                   nnzcolindices = as.integer(nnzcolindices),
-                  lindx = vector("integer",nnzcolindices),     
+                  lindx = vector("integer",nnzcolindices),
                   xlindx = vector("integer",nrow+1),     #
                   nsuper = vector("integer",1),          #
                   nnzR = as.integer(nnzR),#
                   lnz = vector("double",nnzR),        #
                   xlnz = vector("integer",nrow+1),     #
                   snode = vector("integer",nrow),
-                  xsuper = vector("integer",nrow+1),   
+                  xsuper = vector("integer",nrow+1),
                   cachesize = as.integer(cache),
-                  ierr = 0L,          
+                  ierr = 0L,
                   NAOK = getOption("spam.NAOK"), PACKAGE = "spam")
-    
+
   }
- #### end from above 
+ #### end from above
   if(z$ierr == 1) {
-                                        # all other errors trapped 
+                                        # all other errors trapped
       warning("singularity problem or matrix not positive definite",call.=FALSE)
       logdet$modulus <- NA
    } else{
@@ -1071,10 +1066,10 @@ determinant.spam <- function(x, logarithm = TRUE, pivot = "MMD",method="NgPeyton
   }
 
   attr(logdet$modulus,"logarithm") <- logarithm
-  
+
   logdet$sign <- ifelse(z$ierr == 1,NA,1)
   attr(logdet,"class") <- "det"
-  
+
   return(logdet)
 }
 
@@ -1082,15 +1077,15 @@ determinant.spam.chol.NgPeyton <- function(x, logarithm = TRUE,...)
 {
   logdet <- list()
 
-  
+
   tmp <- sum( log(x@entries[ x@rowpointers[-(x@dimension[1]+1)]]))
   if (logarithm) logdet$modulus <- tmp else logdet$modulus <- exp(tmp)
- 
+
   attr(logdet$modulus,"logarithm") <- logarithm
-  
+
   logdet$sign <- 1
   attr(logdet,"class") <- "det"
-  
+
   return(logdet)
 }
 
@@ -1098,10 +1093,15 @@ determinant.spam.chol.NgPeyton <- function(x, logarithm = TRUE,...)
 setMethod("determinant","spam",               determinant.spam)
 setMethod("determinant","spam.chol.NgPeyton", determinant.spam.chol.NgPeyton)
 
+## The ``Right Thing'' to do :
+## base::det() calls [base::]determinant();
+## our det() should call our determinant() :
+det <- base::det
+environment(det) <- environment()## == asNamespace("Matrix")
 ######################################################################
 ########################################################################
 
-    
+
 "as.matrix.spam.chol.NgPeyton" <- function(x,...){
     ## print("as.matrix.spam.chol.NgPeyton")
     nrow <- x@dimension[1]
@@ -1116,14 +1116,14 @@ setMethod("determinant","spam.chol.NgPeyton", determinant.spam.chol.NgPeyton)
         SS <- .format64()
     else
         SS <- .format32
-    
+
     xcolindices <- .C64('calcja',
                         SIGNATURE = c(rep(SS$signature,7)),
                         nrow,
                         nsuper,
                         x@supernodes,
                         x@colindices,
-                        
+
                         x@colpointers,
                         x@rowpointers,
                         xja = vector_dc( SS$type, nnzR),
@@ -1137,7 +1137,7 @@ setMethod("determinant","spam.chol.NgPeyton", determinant.spam.chol.NgPeyton)
   ##                entries = as.double(x@entries),
   ##                colindices = as.integer(xcolindices),
   ##                rowpointers = as.integer(x@rowpointers),
-  ##                res = vector("double",nrow*nrow),  
+  ##                res = vector("double",nrow*nrow),
   ##                NAOK=getOption("spam.NAOK"),PACKAGE = "spam")$res,
   ##              c(nrow,nrow))      # we preserve dimensions
   ##          )
@@ -1148,9 +1148,9 @@ setMethod("determinant","spam.chol.NgPeyton", determinant.spam.chol.NgPeyton)
                  entries = x@entries,
                  colindices = xcolindices,
                  rowpointers = x@rowpointers,
-                 
+
                  res = vector_dc( "double", nrow*nrow),
-                 
+
                  INTENT = c("r", "r", "r", "r",
                             "w"),
                  NAOK=getOption("spam.NAOK"),
