@@ -5,40 +5,32 @@
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
 # by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Daniel Gerber [ctb], Kaspar Moesinger [ctb],           #
-#    Youcef Saad [ctb] (SPARSEKIT),                         #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines),      #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines)       #
+#    Roman Flury [ctb], Daniel Gerber [ctb],                #
+#    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
 
 
 head.spam <- function(x, n = 6L, m = n, ...) {
   stopifnot(length(n) == 1L, length(m) == 1L)
-   n <- if (n < 0L) 
-        max(nrow(x) + n, 0L)
-    else min(n, nrow(x))
-   m <- if (m < 0L) 
-        max(ncol(x) + m, 0L)
-    else min(m, ncol(x))
+   n <- if (n < 0L)
+        max(dim(x)[1] + n, 0L)
+    else min(n, dim(x)[1])
+   m <- if (m < 0L)
+        max(dim(x)[2] + m, 0L)
+    else min(m, dim(x)[2])
     as.matrix(x[seq_len(n), seq_len(m), drop = FALSE])
 }
 
-tail.spam <- function (x, n = 6L, m = n, addrownums = TRUE,  ...) 
+tail.spam <- function (x, n = 6L, m = n, addrownums = TRUE,  ...)
 {
     stopifnot(length(n) == 1L, length(m) == 1L)
-    nrx <- nrow(x)
-    ncx <- ncol(x)
-    n <- if (n < 0L) 
+    nrx <- dim(x)[1]
+    ncx <- dim(x)[2]
+    n <- if (n < 0L)
         max(nrx + n, 0L)
     else min(n, nrx)
-    m <- if (m < 0L) 
+    m <- if (m < 0L)
         max(ncx + m, 0L)
     else min(m, ncx)
     selr <- seq.int(to = nrx, length.out = n)
@@ -47,7 +39,7 @@ tail.spam <- function (x, n = 6L, m = n, addrownums = TRUE,  ...)
     if (addrownums) {
 #    if (addrownums && is.null(rownames(x))) { # rownames is null by default
 #        rownames(ans) <- paste0("[", selr, ",]") # can be used from R2.15
-#        colnames(ans) <- paste0("[,", selc, "]") 
+#        colnames(ans) <- paste0("[,", selc, "]")
         rownames(ans) <- paste("[", selr, ",]", sep = "")
         colnames(ans) <- paste("[,", selc, "]", sep = "")
       }
